#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/SoDB.h>
#include <Inventor/SoInput.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/Xt/SoXtComponent.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>

int make_connection(char *, int);
void TimerCallback(void *, SoSensor *);

int descriptor;

SoCube *cube;

int main(int argc, char **argv) {
	int i, port = 0;
	char server[100];

/*
 *	Parse command line arguments and create a socket
 *
 */

	for (i=1;i<argc - 1;i++) {
		if (strcmp(*(argv+i),"-s") == 0) {	// Server switch
			i++;
			strcpy(server,*(argv + i));	// Set server to the command line argument
		}
		if (strcmp(*(argv+i),"-p") == 0) {	// Port switch
			i++;
			port = atoi(*(argv + i));	// Set port to the command line argument
		}
	}

	if (!server || port == 0) {
		printf("Usage: %s -s server -p port\n",*argv);
		printf("\t-s server\tserver to connect to on specified port\n");
		printf("\t-p port\t\tport to listen for incomming clients\n");
		exit(0);
	}

	if ((descriptor = make_connection(server, port)) <= 0) {
		printf("Error:  Unable to connect to %s:%d\n",server,port);
		exit(0);
	} else printf("descriptor = %d\n", descriptor);

/*
 *	Arguments Parsed, Socket Created, Code Continues...
 *
 */

	/* Initialize Inventor and Xt; create new viewer */
	Widget window = SoXt::init(*argv);
	SoXtExaminerViewer *viewer = new SoXtExaminerViewer(window);

	/* Parse the inventor file */
	SoInput *SceneInput = new SoInput;
	SceneInput->openFile("gview.iv");
	SoSeparator *root = new SoSeparator;
	root = SoDB::readAll(SceneInput);
	SceneInput->closeFile();

	cube = (SoCube *) SoNode::getByName("cube");

	/* Display SceneGraph in viewer */
	viewer->setSceneGraph(root);
	viewer->show();

	/* Schedule invocation of a function with void* arguments */
	SoTimerSensor *TimerSensor = new SoTimerSensor(TimerCallback, NULL);
	TimerSensor->schedule();

	/* Display viewer window and initialize event handler */
	SoXt::show(window);
	SoXt::mainLoop();
}

void TimerCallback(void *arguments, SoSensor *sensor) {
	int connection = 1;
	int	S2I_i_IL;
	float	S2I_f_IL;
	double	S2I_d_IL;

	/* Read values from the Server */
	read(descriptor, (char *) &S2I_i_IL, sizeof(int));
	read(descriptor, (char *) &S2I_f_IL, sizeof(float));
	read(descriptor, (char *) &S2I_d_IL, sizeof(double));
	printf("%d\t%f\t%f\n", S2I_i_IL, S2I_f_IL, S2I_d_IL);

	/* Modify scene with values from Labview */
	cube->width.setValue((float)	S2I_i_IL);
	cube->height.setValue(		S2I_f_IL);
	cube->depth.setValue((float)	S2I_d_IL);

	/* Check if Labview has disconnected */
	read(descriptor, (char *) &connection, sizeof(int));
	write(descriptor, (char *) &connection, sizeof(int));

	if (connection == 0) {
		close(descriptor);
		exit(0);
	}

	/* Schedule another invocation of the function */
	sensor->schedule();
}
